/***************************************************************************
 * e2cd2.c - generate .BAT file for environment change
 *
 * Copyright (C) 1995 Claus Tondering, ct@login.dknet.dk
 * Copyright (C) 1997 Peeter Joot
 * 
 * This file may be redistributed under the terms of the GNU Public License.
 ***************************************************************************/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <linux/types.h>
#include "linux/stat.h"
#include "linux/ext2_fs.h"

#include "ext2fs/ext2fs.h"
#include "ext2fs/ext2_err.h"

#include "dev.h"
#include "e2err.h"
#include "getopt.h"

extern io_manager nt_io_manager;


/**********************************************************************
 * usage prints usage information and exits
 **********************************************************************/

void
usage()
{
	fprintf(stderr, 
"SYNOPSIS\n"
"\tE2CD2 [-bdh] [-f filepath] pathname\n"
"\n"
"DESCRIPTION\n"
"\tThe pathname must identify an ext2 directory. That directory is\n"
"\tmade the current working directory.\n"
"\n"
"-d\tGenerate a dos compatable script.\n"
"\n"
"-b\tGenerate a bash shell compatable script.\n"
"\n"
"-f filepath\tSpecify the name and path of the file for the script.\n"
"\tThe default is .e2cd.bat\n"
"\n"
"-h\tPrints this message.\n"
"\n"
"NOTE\n"
"\tE2CD is a .BAT file that creates another .BAT file which is\n"
"\tcalled by default .e2cd.bat.  This BAT file is used to modify the \n"
"\tE2CWD environment variable. It then executes .e2cd.bat and deletes it.\n"
"\tActually it doesn't delete it, but that was the idea.  This has \n"
"\tsomething to do with NT file sharing semantics, as it seems NT \n"
"\tthinks it is still in use.\n"
"\n"
"\tFor bash users the equivalent of e2cd.bat is available called \n"
"\te2cd.bashrc which can be included in one's .bashrc.\n"
);
	exit(1);
}


/**********************************************************************
 * main routine
 **********************************************************************/

char * prog_name;

main(int argc, char **argv)
{
	int err, c, make_dos_script = 1;
	char env[80], *pos1, *pos2;
	ext2_filsys fs;
	ino_t ino;
	struct ext2_inode e2ino;
	FILE *ofile;
	e2cwd_t cwd;
	char * outfile = NULL;

	prog_name = argv[0];
	initialize_ext2_error_table();

	while ( (c = getopt(argc, argv, "bdhf:") ) != -1 ) {
		switch ( c ) {
			case 'b' : // bash, ksh, ... compatable script.
				make_dos_script = 0;
				break;
			case 'd' : // dos compatable script.
				make_dos_script = 1;
				break;
			case 'f' :
				outfile = optarg;
				break;
			case 'h' :
			case '?' : 
			default :
				usage();
			}
		}

	if ( optind + 1 != argc ) 
		usage();
	
        err = cwdenv_lookup(&cwd);
        if (err)
                com_err("e2cat",0,"E2CWD environment has bad format.",0);

	/* Open file system */
	err = ext2fs_open((const char *)&cwd, 0, 0, 0, nt_io_manager, &fs);
	if (err)
		e2_err(EXT2_ET_CANT_OPEN_FS,err);

	/* Lookup specified name */
	err = ext2fs_namei(fs, 2, cwd.cwdino, argv[argc-1], &ino);
	if (err)
		e2_err(EXT2_ET_NO_FILE,err);

	/* Read specified inode */
	err = ext2fs_read_inode(fs, ino, &e2ino);
	if (err)
		e2_err(EXT2_ET_CANT_READ_IN_INFO,err);

	/* Is it a directory? */
	if (!S_ISDIR(e2ino.i_mode)) {
		fprintf(stderr,"%s is not a directory\n",optarg);
		exit(1);
	}

	if ( !outfile ) 
		outfile = ".e2cd.bat";

	ofile=fopen(outfile,"w");
	if (!ofile) {
		fprintf(stderr,"Cannot create %s\n",outfile);
		exit(1);
	}

	/* We know E2CWD exists, ext2fs_open already checked that */
	strcpy(env,getenv("E2CWD")); /* We are going to modify it, so we make a copy */
	pos1=strchr(env,':');
	pos2=strchr(pos1+1,':');
	if (pos2)
		*pos2=0;

	if ( make_dos_script ) 
		fprintf(ofile,"SET E2CWD=%s:%d\n",env,ino);
	else 
		fprintf(ofile,"export E2CWD=%s:%d\n",env,ino);
	fclose(ofile);
	
	return 0;
}
